function get_state()
  local entity_id = GetUpdatedEntityID()
  state = state or {}
  state[entity_id] = state[entity_id] or {}
  return state[entity_id]
end

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform( entity_id )

local state = get_state()
state.offset_x = state.offset_x or 0
state.offset_y = state.offset_y or 0

x = x + state.offset_x
y = y + state.offset_y

local in_water, hit_x, hit_y = RaytraceSurfacesAndLiquiform( x, y, x, y )
local in_solid, hit_x2, hit_y2 = RaytracePlatforms( x, y, x, y )
local in_particle, hit_x3, hit_y3 = Raytrace( x, y, x, y )

setting_cache = setting_cache or {}
local function get_setting(name)
    setting_cache[name] = setting_cache[name] or ModSettingGet(name)
    return setting_cache[name]
end

local ants = get_setting("particle_life.ants")
local water_bugs = get_setting("particle_life.water_bugs")
local earth_worms = get_setting("particle_life.earth_worms")
local spiders = get_setting("particle_life.spiders")
local flies = get_setting("particle_life.flies")
local fleas = get_setting("particle_life.fleas")
local mycelium = get_setting("particle_life.mycelium")
local dragonflies = get_setting("particle_life.dragonflies")
local fireflies = get_setting("particle_life.fireflies")

if ( in_water and not in_solid ) and ( water_bugs > 0 ) then
  SetRandomSeed( GameGetFrameNum(), x + y )
  local random_number = Random(water_bugs,4)
  if random_number == water_bugs then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/water_bug_water_checker.xml", hit_x4, hit_y4 - 30)
			EntityLoad("mods/particle_life/files/water_bug_water_swamp_checker.xml", hit_x4, hit_y4)
		end
	else
		EntityLoad("mods/particle_life/files/water_bug_water_checker.xml", desired_x, desired_y)
		EntityLoad("mods/particle_life/files/water_bug_water_swamp_checker.xml", desired_x, desired_y)
	end
  end
elseif in_solid and ( earth_worms > 0 ) then
  SetRandomSeed( GameGetFrameNum(), x + y )
  local random_number = Random(earth_worms,15)
  if random_number == earth_worms then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/earth_worm_earth_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/earth_worm_earth_checker.xml", desired_x, desired_y)
	end
  end
elseif ( not in_particle ) then
  SetRandomSeed( GameGetFrameNum(), x + y )
  local spiders_random_number = Random(spiders,20)
  local ants_random_number = Random(ants,40)
  local flies_random_number = Random(flies,40)
  local fleas_random_number = Random(fleas,30)
  local mycelium_random_number = Random(mycelium,60)
  local dragonflies_random_number = Random(dragonflies,60)
  local fireflies_random_number = Random(fireflies,60)

  if spiders_random_number == spiders and ( spiders > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/spider_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/spider_air_checker.xml", desired_x, desired_y)
	end
  end
  if ants_random_number == ants and ( ants > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/ants_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/ants_air_checker.xml", desired_x, desired_y)
	end
  end
  if flies_random_number == flies and ( flies > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/flies_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/flies_air_checker.xml", desired_x, desired_y)
	end
  end
  if fleas_random_number == fleas and ( fleas > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/fleas_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/fleas_air_checker.xml", desired_x, desired_y)
	end
  end
  if mycelium_random_number == mycelium and ( mycelium > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/mycelium_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/mycelium_air_checker.xml", desired_x, desired_y)
	end
  end
 
  if mycelium_random_number == mycelium and ( mycelium > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/mycelium_ceiling_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/mycelium_ceiling_air_checker.xml", desired_x, desired_y)
	end
  end
  if dragonflies_random_number == dragonflies and ( dragonflies > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)

	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/dragonfly_water_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/dragonfly_water_checker.xml", desired_x, desired_y)
	end
  end
  if fireflies_random_number == fireflies and ( fireflies > 0 ) then
	local desired_x = x + Random(-16, 16)
	local desired_y = y + Random(-16, 16)
	if BiomeMapGetName( desired_x, desired_y ) == "_EMPTY_" then
		local ground_below, hit_x4, hit_y4 = RaytraceSurfacesAndLiquiform( desired_x, desired_y, desired_x, desired_y + 50 )
		if ground_below then
			EntityLoad("mods/particle_life/files/firefly_air_checker.xml", hit_x4, hit_y4 - 30)
		end
	else
		EntityLoad("mods/particle_life/files/firefly_air_checker.xml", desired_x, desired_y)
	end
  end
end

if state.offset_y + 16 < 511 then
  if state.offset_x + 16 < 511 then
	state.offset_x = state.offset_x + 16
  else
	state.offset_x = 0
	state.offset_y = state.offset_y + 16
  end
else
  EntityKill(entity_id)
end