local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)

function add_particle_emitter(material, count)
	local hitbox_components = EntityGetComponentIncludingDisabled(entity_id, "HitboxComponent")
	for _, comp in ipairs(hitbox_components) do
		local x_min = ComponentGetValue2( comp, "aabb_min_x" )
		local x_max = ComponentGetValue2( comp, "aabb_max_x" )
		local y_min = ComponentGetValue2( comp, "aabb_min_y" )
		local y_max = ComponentGetValue2( comp, "aabb_max_y" )
		
		local area = (x_max - x_min) * (y_max - y_min)
		local spawn_count = math.ceil(area / 10)
		
		local particle_comp = EntityAddComponent2(entity_id, "ParticleEmitterComponent", {
			emitted_material_name=material,
			x_pos_offset_min=x_min,
			y_pos_offset_min=y_min,
			x_pos_offset_max=x_max,
			y_pos_offset_max=y_max,
			x_vel_min=0,
			x_vel_max=0,
			y_vel_min=0,
			y_vel_max=0,
			count_min=0,
			count_max=1,
			lifetime_min=0.6,
			lifetime_max=1.3,
			create_real_particles=true,
			emit_cosmetic_particles=false,
			emission_interval_min_frames=0,
			emission_interval_max_frames=spawn_count / count,
			is_emitting=true
		} )
	end
end

local name = EntityGetName(entity_id)
local path = EntityGetFilename(entity_id)

if name == "$animal_acidshooter" or
path == "data/entities/animals/vault/acidshooter.xml" or
path == "data/entities/animals/crypt/acidshooter.xml" or
name == "$animal_alchemist" or
name == "$animal_ant" or
name == "$animal_barfer" or
path == "data/entities/animals/crypt/barfer.xml" or
name == "$animal_bat" or
name == "$animal_big_bat" or
name == "$animal_blob" or
path == "data/entities/animals/vault/blob.xml" or
name == "$animal_failed_alchemist_b" or
path == "data/entities/animals/crypt/failed_alchemist_b.xml" or
name == "$animal_frog_big" or
name == "$animal_fungus" or
path == "data/entities/animals/rainforest/fungus.xml" or
name == "$animal_fungus_tiny" or
name == "$animal_lasershooter" or
path == "data/entities/animals/vault/lasershooter.xml" or
name == "$animal_shaman" or
name == "$animal_sheep" or
name == "$animal_sheep_fly" or
name == "$animal_sheep_bat" or
name == "$animal_tentacler" or
path == "data/entities/animals/vault/tentacler.xml" or
path == "data/entities/animals/crypt/tentacler.xml" or
name == "$animal_tentacler_small" or
path == "data/entities/animals/vault/tentacler_small.xml" or
path == "data/entities/animals/crypt/tentacler_small.xml" or
name == "$animal_slimeshooter" then
	add_particle_emitter("particle_life_mosquito_temp", 0.5)
elseif name == "$animal_bigzombie" or
name == "$animal_enlightened_alchemist" or
name == "$animal_failed_alchemist" or
name == "$animal_ghoul" or
name == "$animal_rat" or
name == "$animal_maggot" or
path == "data/entities/animals/vault/bigzombie.xml" or
path == "data/entities/animals/crypt/maggot.xml" or
path == "data/entities/animals/vault/maggot.xml" or
path == "data/entities/animals/crypt/failed_alchemist.xml" or
path == "data/entities/animals/crypt/enlightened_alchemist.xml" or
name == "$animal_zombie" then
	add_particle_emitter("particle_life_mosquito_temp", 0.5)
	add_particle_emitter("particle_life_flea_temp", 0.5)
elseif name == "$animal_shooterflower" or
name == "$animal_fungus_big" or
name == "$animal_fungus_giga" or
name == "$animal_giantshooter" or
name == "$animal_giantshooter_weak" or
path == "data/entities/animals/rainforest/shooterflower.xml" or
path == "data/entities/animals/rainforest/bloom.xml" or
name == "$animal_bloom" then
	add_particle_emitter("particle_life_mosquito_temp", 1)
elseif name == "$animal_deer" or
name == "$animal_elk" then
	add_particle_emitter("particle_life_flea_temp", 0.2)
elseif path == "data/entities/animals/crypt/skullrat.xml" or
path == "data/entities/animals/crypt/skullfly.xml" or
name == "$animal_skullrat" or
name == "$animal_skullfly" then
	add_particle_emitter("particle_life_mosquito_temp", 2)
	add_particle_emitter("particle_life_flea_temp", 2)
end
