dofile_once("data/scripts/lib/utilities.lua")

-- Function to rotate a vector (x, y) by an angle (in radians)
function vec_rotate(x, y, angle)
	local c = math.cos(angle)
	local s = math.sin(angle)
	local new_x = x * c - y * s
	local new_y = x * s + y * c
	return new_x, new_y
end

-- Function to compute points on the edges of a rotated rectangle
function compute_edge_points(x, y, rotation, w, h)
	local corners = {
		{ w, -h },
		{ -w, -h },
		{ -w, h },
		{ w, h }
	}

	local edge_points = {}

	for i = 1, 4 do
		local start_x, start_y = corners[i][1], corners[i][2]
		local end_x, end_y
		if i < 4 then
			end_x, end_y = corners[i + 1][1], corners[i + 1][2]
		else
			end_x, end_y = corners[1][1], corners[1][2]
		end

		-- Number of points on the edge (adjust this for granularity)
		local num_points = 10  -- Adjust as needed

		for j = 0, num_points do
			local fraction = j / num_points
			local edge_x = start_x + (end_x - start_x) * fraction
			local edge_y = start_y + (end_y - start_y) * fraction

			-- Rotate the edge point by the rectangle's rotation
			local rotated_x, rotated_y = vec_rotate(edge_x, edge_y, rotation)
			local final_x = x + rotated_x
			local final_y = y + rotated_y

			table.insert(edge_points, { final_x, final_y })
		end
	end

	return edge_points
end

-- Get entity transform and components
local frame = GameGetFrameNum()
local times_executed = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")
local entity_id = GetUpdatedEntityID()
local x, y, rotation = EntityGetTransform(entity_id)

-- Retrieve VariableStorageComponent values
local components = EntityGetComponent(entity_id, "VariableStorageComponent")

if components ~= nil then
	for _, comp_id in pairs(components) do
		local var_name = ComponentGetValue(comp_id, "name")
		if var_name == "torque" then
			torque_var_comp = comp_id
		elseif var_name == "momentum_x" then
			momentum_x_var_comp = comp_id
		elseif var_name == "momentum_y" then
			momentum_y_var_comp = comp_id
		elseif var_name == "velocity_x" then
			velocity_x_var_comp = comp_id
		elseif var_name == "velocity_y" then
			velocity_y_var_comp = comp_id
		elseif var_name == "gravity" then
			gravity_var_comp = comp_id
		elseif var_name == "radius_x" then
			radius_x_var_comp = comp_id
		elseif var_name == "radius_y" then
			radius_y_var_comp = comp_id
		elseif var_name == "angular_velocity" then
			angular_velocity_var_comp = comp_id
		elseif var_name == "smooth_x" then
			smooth_x_var_comp = comp_id
		elseif var_name == "smooth_y" then
			smooth_y_var_comp = comp_id
		end
	end
end

-- Initialize variables for torque, momentum, and angular velocity
local torque = 0
local momentum_x = 0
local momentum_y = 0
local vel_x1 = 0
local vel_y1 = 0
local gravity = 0
local angular_velocity = 0
local smooth_x, smooth_y = x, y

-- Retrieve values from VariableStorageComponent if available
if gravity_var_comp ~= nil and gravity_var_comp > 0 then
	gravity = ComponentGetValue2(gravity_var_comp, "value_float")
end

if velocity_x_var_comp ~= nil and velocity_x_var_comp > 0 then
	vel_x1 = ComponentGetValue2(velocity_x_var_comp, "value_float")
end

if velocity_y_var_comp ~= nil and velocity_y_var_comp > 0 then
	vel_y1 = ComponentGetValue2(velocity_y_var_comp, "value_float")
end

if radius_x_var_comp ~= nil and radius_x_var_comp > 0 then
	radius_x = ComponentGetValue2(radius_x_var_comp, "value_float")
end

if radius_y_var_comp ~= nil and radius_y_var_comp > 0 then
	radius_y = ComponentGetValue2(radius_y_var_comp, "value_float")
end

if torque_var_comp ~= nil and torque_var_comp > 0 then
	torque = ComponentGetValue2(torque_var_comp, "value_float")
end

if momentum_x_var_comp ~= nil and momentum_x_var_comp > 0 then
	momentum_x = ComponentGetValue2(momentum_x_var_comp, "value_float")
end

if momentum_y_var_comp ~= nil and momentum_y_var_comp > 0 then
	momentum_y = ComponentGetValue2(momentum_y_var_comp, "value_float")
end

if smooth_x_var_comp ~= nil and smooth_x_var_comp > 0 then
	smooth_x = ComponentGetValue2(smooth_x_var_comp, "value_float")
	if ( smooth_x == nil ) or ( smooth_x == 0 ) then
		smooth_x = x
	end
end

if smooth_y_var_comp ~= nil and smooth_y_var_comp > 0 then
	smooth_y = ComponentGetValue2(smooth_y_var_comp, "value_float")
	if ( smooth_y == nil ) or ( smooth_y == 0 ) then
		smooth_y = y
	end
end

if angular_velocity_var_comp ~= nil and angular_velocity_var_comp > 0 then
	angular_velocity = ComponentGetValue2(angular_velocity_var_comp, "value_float")
end

local x, y, rotation = EntityGetTransform(entity_id)

-- vel_y1 = vel_y1 + 0.03
y1 = smooth_y + vel_y1
x1 = smooth_x + vel_x1

-- Function to perform raytrace and resolve collision
function resolve_collision(entity, ex, ey, point_x, point_y, additional_vel_x, additional_vel_y)
	local did_hit, hit_x, hit_y = RaytraceSurfaces(ex, ey, point_x, point_y)

	if did_hit then
		-- Calculate direction from hit point back towards entity center
		local dir_x = ex - hit_x
		local dir_y = ey - hit_y
		local length = math.sqrt(dir_x * dir_x + dir_y * dir_y)
		
		-- Normalize direction vector
		if length > 0 then
			dir_x = dir_x / length
			dir_y = dir_y / length
		end
		
		-- Apply opposing velocity (adjust magnitude as needed)
		local new_vel_x = dir_x * 0.4
		local new_vel_y = dir_y * 0.4
		
		-- Calculate torque based on collision (adjust magnitude as needed)
		local collision_torque = dir_x * 0.005  -- Example torque calculation
		
		-- Adjust angular velocity based on collision torque
		angular_velocity = angular_velocity + collision_torque
		
		-- Return new velocity components
		return new_vel_x , new_vel_y
	else
		return 0, 0, false
	end
end

-- Compute edge points and resolve collisions for each
local w = 3  -- half width
local h = 3  -- half height
if radius_x >= 1 then
	w = radius_x
end
if radius_y >= 1 then
	h = radius_y
end

local edge_points = compute_edge_points(smooth_x, smooth_y, rotation, w, h)
local additional_vel_x, additional_vel_y = 0, 0

for i, point in ipairs(edge_points) do
	-- GameCreateSpriteForXFrames("mods/particle_life/files/plants/debug_indicator7.png", point[1], point[2], true, 0, 0, 1, true)
	
	vel_x2, vel_y2 = resolve_collision(entity_id, x1, y1, point[1], point[2], additional_vel_x, additional_vel_y)
	additional_vel_x = additional_vel_x + vel_x2
	additional_vel_y = additional_vel_y + vel_y2
end

if ( additional_vel_x > 0.2 ) or ( additional_vel_y > 0.2 ) then
	if times_executed % 10 == 0 then
		GamePlaySound( "data/audio/Desktop/materials.snd", "collision/sand", x, y )
	end
end
local gravity = 0.03
local angular_damping = 0.95
-- Update the entity transform with the new position and rotation
local friction_y = 1
local friction_x = 1
if additional_vel_x ~= 0 then
	friction_x = 0.3
	if ( additional_vel_x < 0.2 ) and ( additional_vel_x > -0.2 ) then
		friction_x = 0
		angular_damping = 0
	end
end

if additional_vel_y ~= 0 then
	friction_y = 0.1
	if ( additional_vel_y < 0.2 ) and ( additional_vel_y > -0.2 ) then
		friction_y = 0
		angular_damping = 0
	end
end

-- GamePrint(friction_x)
-- GamePrint(friction_y)

EntitySetTransform(entity_id, math.floor( x1 + ( ( vel_x1 + additional_vel_x ) * friction_x ) + 0.5 ), math.floor( y1 + ( ( vel_y1 + additional_vel_y + gravity) * friction_y ) + 0.5 ), rotation + angular_velocity)

-- Apply friction to linear velocities
ComponentSetValue2(velocity_x_var_comp, "value_float", ( vel_x1 + additional_vel_x ) * friction_x )
ComponentSetValue2(velocity_y_var_comp, "value_float", ( vel_y1 + additional_vel_y + gravity ) * friction_y )

ComponentSetValue2(smooth_x_var_comp, "value_float", x1 + ( ( vel_x1 + additional_vel_x ) * friction_x ) )
ComponentSetValue2(smooth_y_var_comp, "value_float", y1 + ( ( vel_y1 + additional_vel_y + gravity ) * friction_y ) )

-- Apply angular damping

angular_velocity = angular_velocity * angular_damping

-- Store the updated angular velocity
if angular_velocity_var_comp ~= nil and angular_velocity_var_comp > 0 then
	ComponentSetValue2(angular_velocity_var_comp, "value_float", angular_velocity)
end

local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x, y)
if did_hit then
	GamePrint("Fighting it")
	ComponentSetValue2(smooth_x_var_comp, "value_float", smooth_x )
	ComponentSetValue2(smooth_y_var_comp, "value_float", smooth_y - 1 )
	ComponentSetValue2(velocity_x_var_comp, "value_float", 0 )
	ComponentSetValue2(velocity_y_var_comp, "value_float", 0 )
	EntitySetTransform(entity_id, x, y - 1)
	return
end