function collision_trigger()	
	function calculate_distance(x1, y1, x2, y2)
		return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
	end
	
    local entity_id = GetUpdatedEntityID()
    local times = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")
    local x, y, rot = EntityGetTransform(entity_id)
	
	-- Function to rotate a vector (x, y) by an angle (in radians)
	function vec_rotate(x, y, angle)
		local c = math.cos(angle)
		local s = math.sin(angle)
		local new_x = x * c - y * s
		local new_y = x * s + y * c
		return new_x, new_y
	end
	
	-- Retrieve VariableStorageComponent values
	local components = EntityGetComponent(entity_id, "VariableStorageComponent")
	
	local idle_frames = 0

	if components ~= nil then
		for _, comp_id in pairs(components) do
			local var_name = ComponentGetValue(comp_id, "name")
			if var_name == "torque" then
				torque_var_comp = comp_id
			elseif var_name == "momentum_x" then
				momentum_x_var_comp = comp_id
			elseif var_name == "momentum_y" then
				momentum_y_var_comp = comp_id
			elseif var_name == "velocity_x" then
				velocity_x_var_comp = comp_id
			elseif var_name == "velocity_y" then
				velocity_y_var_comp = comp_id
			elseif var_name == "gravity" then
				gravity_var_comp = comp_id
			elseif var_name == "radius_x" then
				radius_x_var_comp = comp_id
			elseif var_name == "radius_y" then
				radius_y_var_comp = comp_id
			elseif var_name == "angular_velocity" then
				angular_velocity_var_comp = comp_id
			elseif var_name == "smooth_x" then
				smooth_x_var_comp = comp_id
			elseif var_name == "smooth_y" then
				smooth_y_var_comp = comp_id
			elseif var_name == "projectiles" then
				projectiles_var_comp = comp_id
			elseif var_name == "idle_frames" then
				idle_frames_var_comp = comp_id
			end
		end
	end
	
	--Getting all projectiles and their explosions to simulate digging
	if projectiles_var_comp ~= nil and projectiles_var_comp > 0 then

		function split_string(input_str, delimiter)
			local result = {}
			for match in (input_str .. delimiter):gmatch("(.-)" .. delimiter) do
				table.insert(result, match)
			end
			return result
		end
		
		function decode_stored_coords(val)
			local out = {}
			local stored_coords = split_string(val, ",")
			for i=1, #stored_coords, 10 do
				table.insert(out, {id = stored_coords[i], radius = tonumber(stored_coords[i+1]), x = tonumber(stored_coords[i+2]), y = tonumber(stored_coords[i+3])})
			end
			return out
		end
		
		function encode_coords_to_store(t)
			local result = {}
			for _, data in ipairs(t) do
				table.insert(result, data.id)
				table.insert(result, tostring(data.radius))
				table.insert(result, tostring(data.x))
				table.insert(result, tostring(data.y))
			end
			return table.concat(result, ",")
		end
		
		-- Retrieve previous projectiles
		local val = ComponentGetValue2(projectiles_var_comp, "value_string")
		local previous_projectiles = decode_stored_coords(val)
		
		local projectiles = {}
		local entities = EntityGetInRadius(x, y, 40)
		local max_projectiles = 1
		
		-- Calculate priority score and collect relevant projectiles
		for _, entity in ipairs(entities) do
			local projectile_component = EntityGetFirstComponent(entity, "ProjectileComponent")
			if projectile_component ~= nil and projectile_component > 0 then
				local on_death_explodes = ComponentGetValue2(projectile_component, "on_death_explode")
				local on_lifetime_out_explodes = ComponentGetValue2(projectile_component, "on_lifetime_out_explode")
				if on_death_explodes == true or on_lifetime_out_explodes == true then
					local explosion_radius = ComponentObjectGetValue2(projectile_component, "config_explosion", "explosion_radius")
					local px, py = EntityGetTransform(entity)
					local distance = math.sqrt((px - x) ^ 2 + (py - y) ^ 2)
					local priority_score = distance - explosion_radius
					table.insert(projectiles, {id = entity, radius = explosion_radius, x = px, y = py, score = priority_score})
				end
			end
		end

		-- Sort projectiles based on priority score (lower score is higher priority)
		table.sort(projectiles, function(a, b) return a.score < b.score end)

		-- Keep only the top projectiles up to the max limit
		local prioritized_projectiles = {}
		for i = 1, math.min(max_projectiles, #projectiles) do
			table.insert(prioritized_projectiles, projectiles[i])
		end

		-- Function to check if explosion overlaps with plant corners
		function check_explosion_overlap(explosion_x, explosion_y, explosion_radius, offsets)
			for i=1, #offsets, 2 do
				local offset_x = offsets[i]
				local offset_y = offsets[i + 1]
				local pos_x = x - offset_x
				local pos_y = y - offset_y
				
				-- GameCreateSpriteForXFrames("mods/particle_life/files/plants/debug_indicator7.png", pos_x, pos_y, true, 0, 0, 1, true)

				local distance = calculate_distance(pos_x, pos_y, explosion_x, explosion_y)
				if distance <= explosion_radius then
					return true
				end
			end

			return false
		end
		
		-- Check for expired projectiles
		local sprite_offsets = {}
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		for i, comp in ipairs(sprite_comps) do
			local offset_x = ComponentGetValue2(comp, "offset_x")
			local offset_y = ComponentGetValue2(comp, "offset_y")
			table.insert(sprite_offsets, offset_x)
			table.insert(sprite_offsets, offset_y)
		end
		for _, prev_proj in ipairs(previous_projectiles) do
			if not EntityGetIsAlive(tonumber(prev_proj.id)) then
				if prev_proj.radius ~= nil then
					-- GamePrint("Yes, explosion radius was: " .. tostring(prev_proj.radius).. " " .. tostring(prev_proj.x).. " " .. tostring(prev_proj.y))
					
					local overlap = check_explosion_overlap(tonumber(prev_proj.x), tonumber(prev_proj.y), tonumber(prev_proj.radius), sprite_offsets)
					if overlap == true then
						EntityKill(entity_id)
					end
				end
			end
		end
		
		-- Update the VariableStorageComponent with the new list of active projectiles
		local encode_string = encode_coords_to_store(prioritized_projectiles)
		ComponentSetValue2(projectiles_var_comp, "value_string", encode_string)
	end
	
	if ( idle_frames_var_comp ~= nil ) and ( idle_frames_var_comp > 0 ) then
		idle_frames = ComponentGetValue2(idle_frames_var_comp, "value_int")
	end
	
	function calculate_distance(x1, y1, x2, y2)
		return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
	end

	-- Function to normalize a vector
	local function normalize_vector(dx, dy)
		local length = math.sqrt(dx * dx + dy * dy)
		if length > 0 then
			return dx / length, dy / length
		else
			return 0, 0
		end
	end
	
	-- Function to calculate the torque
	local function calculate_torque(px, py, ex, ey, force_x, force_y)
		local r_x = ex - px
		local r_y = ey - py
		return r_x * force_y - r_y * force_x
	end

	
	-- EntitySetTransform( entity_id, math.floor(x + 0.5) , math.floor(y + 0.5) )
	local is_being_kicked = false
	local players = EntityGetInRadiusWithTag(x, y, 50, "player_unit")
	local player_id = 0
	if #players > 0 then
		player_id = players[1]
		local px, py = EntityGetTransform(player_id)
		local player_shooter_component = EntityGetFirstComponent(player_id, "PlatformShooterPlayerComponent")
		local controls_component = EntityGetFirstComponent(player_id, "ControlsComponent")
		local character_data_component = EntityGetFirstComponent(player_id, "CharacterDataComponent")
		if ( controls_component ~= nil ) and ( controls_component > 0 ) then
			-- local button_kick = ComponentGetValue2(controls_component, "mButtonDownKick")
			local kick_frame = ComponentGetValue2(controls_component, "mButtonFrameKick")
			local current_frame = GameGetFrameNum()
			if ( current_frame > kick_frame + 20 ) and ( current_frame < kick_frame + 25 ) then
				local kick_component = EntityGetFirstComponent(player_id, "KickComponent")
				if ( kick_component ~= nil ) and ( kick_component > 0 ) then
					local kick_radius = ComponentGetValue2(kick_component, "kick_radius")
					kick_radius = kick_radius + 10
					local kick_force = ComponentGetValue2(kick_component, "player_kickforce")
					local distance = calculate_distance(px, py, x, y)
					if distance < kick_radius then
						is_being_kicked = true
						local mx, my = DEBUG_GetMouseWorld()
						
						-- Calculate the direction vector from (x, y) to (mx, my)
						local dir_x, dir_y = normalize_vector(mx - x, my - y)
						
						-- Set the magnitude of the force
						local force_magnitude = kick_force  -- Adjust this value as needed
						
						-- Calculate the velocity components
						local vel_x = dir_x * force_magnitude
						local vel_y = dir_y * force_magnitude
						
						-- Set the velocity components
						ComponentSetValue2(velocity_x_var_comp, "value_float", vel_x * 0.05)
						ComponentSetValue2(velocity_y_var_comp, "value_float", vel_y * 0.05)
						
						local torque = calculate_torque(px, py, x, y, vel_x, vel_y)
						
						-- Retrieve and update the angular velocity
						local angular_velocity = ComponentGetValue2(angular_velocity_var_comp, "value_float")
						angular_velocity = angular_velocity + torque * 0.001
						
						GamePlaySound( "data/audio/Desktop/materials.snd", "collision/sand", x, y )
						
						-- Store the updated angular velocity
						if angular_velocity_var_comp ~= nil and angular_velocity_var_comp > 0 then
							ComponentSetValue2(angular_velocity_var_comp, "value_float", angular_velocity)
						end
					end
				end
			end
			local is_eating = false
			local button_down = ComponentGetValue2(controls_component, "mButtonDownDown")
			if ( character_data_component ~= nil ) and ( character_data_component > 0 ) and ( button_down == true ) then
				local is_on_ground = ComponentGetValue2(character_data_component, "is_on_ground")
				if is_on_ground == true then
					is_eating = true
				end
			end
			if ( player_shooter_component ~= nil ) and ( player_shooter_component > 0 ) and ( is_eating == true ) then
				local x_min, y_min = ComponentGetValue2(player_shooter_component, "eating_area_min")
				local x_max, y_max = ComponentGetValue2(player_shooter_component, "eating_area_max")
				
				y_min = y_min - 5
				
				local pixel_components = EntityGetComponentIncludingDisabled(entity_id, "SpriteComponent")
				if pixel_components then
					-- Create a table to hold the new anchor positions and textures
					local skewed_pixels = {}
					local is_in_area = false

					for i, pixel_comp in ipairs(pixel_components) do
						local pos_x = x - ComponentGetValue2(pixel_comp, "offset_x")
						local pos_y = y - ComponentGetValue2(pixel_comp, "offset_y")
						if pos_x >= (px + x_min) and pos_x <= (px + x_max) and pos_y >= (py + y_min) and pos_y <= (py + y_max) then
							is_in_area = true
							break
						end
					end
					
					if is_in_area == true then
						for i, pixel_comp in ipairs(pixel_components) do
							local anchor_x = ComponentGetValue2(pixel_comp, "offset_x")
							local anchor_y = ComponentGetValue2(pixel_comp, "offset_y")
							EntityLoad( "mods/particle_life/files/plants/plant_corpse.xml", x - anchor_x , y - anchor_y )
						end
						EntityKill(entity_id)
						return
					end
				end
			end
		end
	end
	
	if radius_x_var_comp ~= nil and radius_x_var_comp > 0 then
		radius_x = ComponentGetValue2(radius_x_var_comp, "value_float")
	end

	if radius_y_var_comp ~= nil and radius_y_var_comp > 0 then
		radius_y = ComponentGetValue2(radius_y_var_comp, "value_float")
	end
	
	local cx1, cy1 = vec_rotate(-radius_x, radius_y, rot)
	local cx2, cy2 = vec_rotate(radius_x, -radius_y, rot)
	local cx3, cy3 = vec_rotate(-radius_x, -radius_y, rot)
	local cx4, cy4 = vec_rotate(radius_x, radius_y, rot)

	if idle_frames > 60 then
		
		local did_hit1 = RaytraceSurfaces(x + cx1, y + cy1, x + cx2, y + cy2)
		local did_hit2 = RaytraceSurfaces(x + cx2, y + cy2, x + cx3, y + cy3)
		local did_hit3 = RaytraceSurfaces(x + cx3, y + cy3, x + cx4, y + cy4)
		local did_hit4 = RaytraceSurfaces(x + cx4, y + cy4, x + cx1, y + cy1)

		
		-- local did_hit1 = RaytraceSurfaces(x, y, x, y + 5)
		if ( did_hit1 or did_hit2 or did_hit3 or did_hit4 ) and ( is_being_kicked == false ) then
			return
		else
			ComponentSetValue2(idle_frames_var_comp, "value_int", 0)
		end
	end

    function shear_x(px, py, angle)
        local new_x = px + py * (math.tan(angle / 2) * -1)
        return math.floor(new_x + 0.5), py  -- Round to nearest whole number
    end

    function shear_y(px, py, angle)
        local new_y = py + px * math.sin(angle)
        return px, math.floor(new_y + 0.5)  -- Round to nearest whole number
    end

    function rotate(px, py, angle)
        local cos_a = math.cos(angle)
        local sin_a = math.sin(angle)
        local new_x = px * cos_a - py * sin_a
        local new_y = px * sin_a + py * cos_a
        return math.floor(new_x + 0.5), math.floor(new_y + 0.5)
    end

	function get_quadrant(angle)
	  while angle < 0 do
		angle = angle + math.pi * 2
	  end
	  for i=1, 4 do
		if angle < math.pi / 2 then
		  return i
		end
		angle = angle - math.pi / 2
	  end
	  return 1
	end

    function adjust_angle_to_quadrant(angle)
        return angle % (math.pi / 2)
    end

    function string_split(input, sep)
        local result = {}
        for v in string.gmatch(input .. sep, '([^' .. sep .. ']*)' .. sep) do
            table.insert(result, v)
        end
        return result
    end

    function load_vegetation_pixel2(image_file2, px, py)
        local pixelcomp = EntityAddComponent2(entity_id, "SpriteComponent", {
            image_file = image_file2,
            offset_x = px,
            offset_y = py,
			update_transform_rotation = false,
			has_special_scale = true,
			special_scale_x = 1,
			special_scale_y = 1
        })
        ComponentAddTag(pixelcomp, "enabled_in_world")
    end

    local imprint_var_comp_old = 0
    local components = EntityGetComponent(entity_id, "VariableStorageComponent")
    if components ~= nil then
        for _, comp_id in pairs(components) do
            local var_name = ComponentGetValue(comp_id, "name")
            if var_name == "rotation_prev" then
                var_comp_rotation_prev = comp_id
            end
            if var_name == "imprint" then
                imprint_var_comp_old = comp_id
            end
			if var_name == "velocity_x" then
				velocity_x_var_comp = comp_id
			elseif var_name == "velocity_y" then
				velocity_y_var_comp = comp_id
			elseif var_name == "angular_velocity" then
				angular_velocity_var_comp = comp_id
			end
        end
    end

    if ( imprint_var_comp_old == nil ) or ( imprint_var_comp_old == 0 ) then
        -- Initialize an imprint table to store initial positions and textures
        local imprint_table = {}

        -- Iterate through all PixelSpriteComponents and capture initial positions and textures
        local pixel_components = EntityGetComponentIncludingDisabled(entity_id, "SpriteComponent")
        if pixel_components ~= nil and #pixel_components > 0 then
            for _, pixel_comp in ipairs(pixel_components) do
                local anchor_x = ComponentGetValue2(pixel_comp, "offset_x")
                local anchor_y = ComponentGetValue2(pixel_comp, "offset_y")
                local texture = ComponentGetValue2(pixel_comp, "image_file")

                -- Store initial positions and texture in imprint table
                table.insert(imprint_table, texture)
                table.insert(imprint_table, anchor_x)
                table.insert(imprint_table, anchor_y)
            end
        end

        -- Convert imprint table to string
        local imprint_str = table.concat(imprint_table, ",")

        EntityAddComponent2(entity_id, "VariableStorageComponent", {
            name = "imprint",
            value_string = imprint_str
        })
    end

    local components = EntityGetComponent(entity_id, "VariableStorageComponent")
    if components ~= nil then
        for _, comp_id in pairs(components) do
            local var_name = ComponentGetValue(comp_id, "name")
            if var_name == "imprint" then
                imprint_var_comp = comp_id
            end
        end
    end
	
    local rotation_prev = 0
	if ( var_comp_rotation_prev ~= nil ) and ( var_comp_rotation_prev > 0 ) then
		rotation_prev = ComponentGetValue2(var_comp_rotation_prev, "value_float")
	end
    local rot_diff = math.abs(rotation_prev - rot)

    local x, y, rot = EntityGetTransform(entity_id)

    if rot ~= rotation_prev and rot_diff > 0.005 then
        -- Iterate through all PixelSpriteComponents and adjust their anchor points
        local pixel_components = EntityGetComponentIncludingDisabled(entity_id, "SpriteComponent")
        if ( pixel_components ~= nil ) and ( #pixel_components > 0 ) then
            -- Create a table to hold the new anchor positions and textures
            local skewed_pixels = {}

            for i, pixel_comp in ipairs(pixel_components) do
                local anchor_x = ComponentGetValue2(pixel_comp, "offset_x")
                local anchor_y = ComponentGetValue2(pixel_comp, "offset_y")
                local texture = ComponentGetValue2(pixel_comp, "image_file")

                if imprint_var_comp ~= nil and imprint_var_comp > 0 then
                    local values = ComponentGetValue2(imprint_var_comp, "value_string")
                    local value_table = string_split(values, ",")
                    texture = tostring(value_table[1 + (i - 1) * 3])
                    anchor_x = tonumber(value_table[2 + (i - 1) * 3])
                    anchor_y = tonumber(value_table[3 + (i - 1) * 3])
                end

                -- Adjust angle to 0 to pi/2 range
                local adjusted_angle = adjust_angle_to_quadrant(rot)
                
                -- Apply shearing transformations
                local new_anchor_x, new_anchor_y = shear_x(anchor_x, anchor_y, adjusted_angle)
                new_anchor_x, new_anchor_y = shear_y(new_anchor_x, new_anchor_y, adjusted_angle)
                new_anchor_x, new_anchor_y = shear_x(new_anchor_x, new_anchor_y, adjusted_angle)
                
                -- Determine the quadrant and rotate accordingly
                local quadrant = get_quadrant(rot)
                local rotation_angle = (quadrant - 1) * (math.pi / 2)
                new_anchor_x, new_anchor_y = rotate(new_anchor_x, new_anchor_y, rotation_angle)

                -- Store the new anchor positions and texture in the table
                table.insert(skewed_pixels, {texture, new_anchor_x, new_anchor_y})

                -- Remove the original component
                EntityRemoveComponent(entity_id, pixel_comp)
            end

            -- Add the new components with the skewed anchor points
            for _, pixel_data in ipairs(skewed_pixels) do
                load_vegetation_pixel2(pixel_data[1], pixel_data[2], pixel_data[3])
            end
        end
		if ( var_comp_rotation_prev ~= nil ) and ( var_comp_rotation_prev > 0 ) then
			ComponentSetValue2(var_comp_rotation_prev, "value_float", rot)
		end
    end
	
	

	--Physics simulation:

	-- Function to compute points on the edges of a rotated rectangle
	function compute_edge_points(x, y, rotation, w, h)
		local corners = {
			{ w, -h },
			{ -w, -h },
			{ -w, h },
			{ w, h }
		}

		local edge_points = {}

		for i = 1, 4 do
			local start_x, start_y = corners[i][1], corners[i][2]
			local end_x, end_y
			if i < 4 then
				end_x, end_y = corners[i + 1][1], corners[i + 1][2]
			else
				end_x, end_y = corners[1][1], corners[1][2]
			end

			-- Number of points on the edge (adjust this for granularity)
			local num_points = 1  -- Adjust as needed

			for j = 0, num_points do
				local fraction = j / num_points
				local edge_x = start_x + (end_x - start_x) * fraction
				local edge_y = start_y + (end_y - start_y) * fraction

				-- Rotate the edge point by the rectangle's rotation
				local rotated_x, rotated_y = vec_rotate(edge_x, edge_y, rotation)
				local final_x = x + rotated_x
				local final_y = y + rotated_y

				table.insert(edge_points, { final_x, final_y })
			end
		end

		return edge_points
	end

	-- Get entity transform and components
	local frame = GameGetFrameNum()
	local times_executed = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")
	local entity_id = GetUpdatedEntityID()
	local x, y, rotation = EntityGetTransform(entity_id)

	-- Initialize variables for torque, momentum, and angular velocity
	local torque = 0
	local momentum_x = 0
	local momentum_y = 0
	local vel_x1 = 0
	local vel_y1 = 0
	local gravity = 0
	local angular_velocity = 0
	local smooth_x, smooth_y = x, y

	-- Retrieve values from VariableStorageComponent if available
	if gravity_var_comp ~= nil and gravity_var_comp > 0 then
		gravity = ComponentGetValue2(gravity_var_comp, "value_float")
	end

	if velocity_x_var_comp ~= nil and velocity_x_var_comp > 0 then
		vel_x1 = ComponentGetValue2(velocity_x_var_comp, "value_float")
	end

	if velocity_y_var_comp ~= nil and velocity_y_var_comp > 0 then
		vel_y1 = ComponentGetValue2(velocity_y_var_comp, "value_float")
	end

	if radius_x_var_comp ~= nil and radius_x_var_comp > 0 then
		radius_x = ComponentGetValue2(radius_x_var_comp, "value_float")
	end

	if radius_y_var_comp ~= nil and radius_y_var_comp > 0 then
		radius_y = ComponentGetValue2(radius_y_var_comp, "value_float")
	end

	if torque_var_comp ~= nil and torque_var_comp > 0 then
		torque = ComponentGetValue2(torque_var_comp, "value_float")
	end

	if momentum_x_var_comp ~= nil and momentum_x_var_comp > 0 then
		momentum_x = ComponentGetValue2(momentum_x_var_comp, "value_float")
	end

	if momentum_y_var_comp ~= nil and momentum_y_var_comp > 0 then
		momentum_y = ComponentGetValue2(momentum_y_var_comp, "value_float")
	end

	if smooth_x_var_comp ~= nil and smooth_x_var_comp > 0 then
		smooth_x = ComponentGetValue2(smooth_x_var_comp, "value_float")
		if ( smooth_x == nil ) or ( smooth_x == 0 ) then
			smooth_x = x
		end
	end

	if smooth_y_var_comp ~= nil and smooth_y_var_comp > 0 then
		smooth_y = ComponentGetValue2(smooth_y_var_comp, "value_float")
		if ( smooth_y == nil ) or ( smooth_y == 0 ) then
			smooth_y = y
		end
	end

	if angular_velocity_var_comp ~= nil and angular_velocity_var_comp > 0 then
		angular_velocity = ComponentGetValue2(angular_velocity_var_comp, "value_float")
	end
	
	local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x, y)
	if did_hit then
		ComponentSetValue2(smooth_x_var_comp, "value_float", smooth_x )
		ComponentSetValue2(smooth_y_var_comp, "value_float", smooth_y - 1 )
		ComponentSetValue2(velocity_x_var_comp, "value_float", 0 )
		ComponentSetValue2(velocity_y_var_comp, "value_float", 0 )
		EntitySetTransform(entity_id, x, y - 1)
		return
	end

	local x, y, rotation = EntityGetTransform(entity_id)

	-- vel_y1 = vel_y1 + 0.03
	y1 = smooth_y + vel_y1
	x1 = smooth_x + vel_x1

	-- Function to perform raytrace and resolve collision
	function resolve_collision(entity, ex, ey, point_x, point_y, additional_vel_x, additional_vel_y)
		local did_hit, hit_x, hit_y = RaytraceSurfaces(ex, ey, point_x, point_y)
		
		-- function pointInsideRect(x2, y2, rx, ry, rot)
		  -- local nx, ny = vec_rotate(x2 - x, y2 - y, -rot)
		  -- return (nx + x) >= x - rx and (ny + y) >= y - ry and (nx + x) <= x + rx and (ny + y) <= y + ry
		-- end
		-- local colliding_with_plant = false
		-- local plants = EntityGetInRadiusWithTag(x, y, 10, "boss_ghost_lasers")
		-- if #plants > 1 then
			-- for _,plant in pairs(plants) do
				-- if plant ~= entity_id then
					-- local plant_components = EntityGetComponent(plant, "VariableStorageComponent")

					-- if plant_components ~= nil then
						-- for _, comp_id in pairs(plant_components) do
							-- local var_name = ComponentGetValue(comp_id, "name")
							-- if var_name == "radius_x" then
								-- plant_radius_x_var_comp = comp_id
							-- elseif var_name == "radius_y" then
								-- plant_radius_y_var_comp = comp_id
							-- end
						-- end
						-- if plant_radius_x_var_comp ~= nil and plant_radius_x_var_comp > 0 then
							-- plant_radius_x = ComponentGetValue2(plant_radius_x_var_comp, "value_float")
						-- end

						-- if plant_radius_y_var_comp ~= nil and plant_radius_y_var_comp > 0 then
							-- plant_radius_y = ComponentGetValue2(plant_radius_y_var_comp, "value_float")
						-- end
						
						-- local pos_x, pos_y, rot = EntityGetTransform(plant)
						
						-- colliding_with_plant = pointInsideRect(pos_x, pos_y, plant_radius_x, plant_radius_y, rot)
					-- end
				-- end
			-- end
		-- end

		if did_hit then
			-- Calculate direction from hit point back towards entity center
			local dir_x = ex - hit_x
			local dir_y = ey - hit_y
			local length = math.sqrt(dir_x * dir_x + dir_y * dir_y)
			
			-- Normalize direction vector
			if length > 0 then
				dir_x = dir_x / length
				dir_y = dir_y / length
			end
			
			-- Apply opposing velocity (adjust magnitude as needed)
			local new_vel_x = dir_x * 0.65
			local new_vel_y = dir_y * 0.35
			
			-- Calculate torque based on collision (adjust magnitude as needed)
			local collision_torque = dir_x * 0.008  -- Example torque calculation
			
			-- Adjust angular velocity based on collision torque
			angular_velocity = angular_velocity + collision_torque
			
			-- Return new velocity components
			return new_vel_x , new_vel_y
		-- elseif colliding_with_plant then
			-- local dir_x = ex - point_x
			-- local dir_y = ey - point_y
			-- local length = math.sqrt(dir_x * dir_x + dir_y * dir_y)
			
			-- -- Normalize direction vector
			-- if length > 0 then
				-- dir_x = dir_x / length
				-- dir_y = dir_y / length
			-- end
			
			-- -- Apply opposing velocity (adjust magnitude as needed)
			-- local new_vel_x = dir_x * 0.4
			-- local new_vel_y = dir_y * 0.4
			
			-- -- Calculate torque based on collision (adjust magnitude as needed)
			-- local collision_torque = dir_x * 0.005  -- Example torque calculation
			
			-- -- Adjust angular velocity based on collision torque
			-- angular_velocity = angular_velocity + collision_torque
			
			-- -- Return new velocity components
			-- return new_vel_x , new_vel_y
		else
			return 0, 0, false
		end
	end

	-- Compute edge points and resolve collisions for each
	local w = 3  -- half width
	local h = 3  -- half height
	if radius_x >= 1 then
		w = radius_x
	end
	if radius_y >= 1 then
		h = radius_y
	end

	-- local edge_points = compute_edge_points(smooth_x, smooth_y, rotation, w, h)
	-- local rx1, ry1 = vec_rotate(-radius_x, radius_y, rot)
	-- local rx2, ry2 = vec_rotate(radius_x, -radius_y, rot)
	-- local rx3, ry3 = vec_rotate(-radius_x, -radius_y, rot)
	-- local rx4, ry4 = vec_rotate(radius_x, radius_y, rot)
	
	-- local rx5, ry5 = vec_rotate(0, radius_y, rot)
	-- local rx6, ry6 = vec_rotate(0, -radius_y, rot)
	-- local rx7, ry7 = vec_rotate(radius_x, 0, rot)
	-- local rx8, ry8 = vec_rotate(-radius_x, 0, rot)
	
	local edge_points = {
		{ smooth_x + cx1, smooth_y + cy1 },
		{ smooth_x + cx2, smooth_y + cy2 },
		{ smooth_x + cx3, smooth_y + cy3 },
		{ smooth_x + cx4, smooth_y + cy4 }
		-- { smooth_x + rx5, smooth_y + ry5 },
		-- { smooth_x + rx6, smooth_y + ry6 },
		-- { smooth_x + rx7, smooth_y + ry7 },
		-- { smooth_x + rx8, smooth_y + ry8 }
	}
	
	local additional_vel_x, additional_vel_y = 0, 0

	for i, point in ipairs(edge_points) do
		-- GameCreateSpriteForXFrames("mods/particle_life/files/plants/debug_indicator7.png", point[1], point[2], true, 0, 0, 1, true)
		
		vel_x2, vel_y2 = resolve_collision(entity_id, x1, y1, point[1], point[2], additional_vel_x, additional_vel_y)
		additional_vel_x = additional_vel_x + vel_x2
		additional_vel_y = additional_vel_y + vel_y2
	end
	local sound_treshold = 2
	if ( ( additional_vel_x > sound_treshold ) or ( additional_vel_x < sound_treshold * -1 ) ) or ( ( additional_vel_y > sound_treshold ) or ( additional_vel_y < sound_treshold * -1 ) ) then
		if times_executed % 10 == 0 then
			GamePlaySound( "data/audio/Desktop/materials.snd", "collision/sand", x, y )
		end
	end
	local gravity = 0.03
	local angular_damping = 0.95
	-- Update the entity transform with the new position and rotation
	local friction_y = 1
	local friction_x = 1
	local contributes_idle = false
	
	if additional_vel_x ~= 0 then
		friction_x = 0.2
		if ( additional_vel_x < 0.18 ) and ( additional_vel_x > -0.18 ) then
			friction_x = 0.02
			angular_damping = 0
		end
		if ( additional_vel_x < 2 ) and ( additional_vel_x > -2 ) then
			contributes_idle = true
		end
	end

	if additional_vel_y ~= 0 then
		friction_y = 0.2
		if ( additional_vel_y < 0.2 ) and ( additional_vel_y > -0.2 ) then
			friction_y = 0.02
			angular_damping = 0
		end
		if ( additional_vel_y < 2 ) and ( additional_vel_y > -2 ) then
			contributes_idle = true
		end
	end
	if contributes_idle == true then
		ComponentSetValue2(idle_frames_var_comp, "value_int", idle_frames + 1)
	end
	if ( additional_vel_x > 0.3 ) and ( additional_vel_x < -0.3 ) then
		ComponentSetValue2(idle_frames_var_comp, "value_int", 0)
	elseif ( additional_vel_x > 0.8 ) and ( additional_vel_x < -0.8 ) then
		ComponentSetValue2(idle_frames_var_comp, "value_int", 0)
	end
	
	if is_being_kicked then
		ComponentSetValue2(idle_frames_var_comp, "value_int", 0)
	end

	-- GamePrint(friction_x)
	-- GamePrint(friction_y)

	EntitySetTransform(entity_id, math.floor( x1 + ( ( vel_x1 + additional_vel_x ) * friction_x ) + 0.5 ), math.floor( y1 + ( ( vel_y1 + additional_vel_y + gravity) * friction_y ) + 0.5 ), rotation + angular_velocity)

	-- Apply friction to linear velocities
	ComponentSetValue2(velocity_x_var_comp, "value_float", ( vel_x1 + additional_vel_x ) * friction_x )
	ComponentSetValue2(velocity_y_var_comp, "value_float", ( vel_y1 + additional_vel_y + gravity ) * friction_y )

	ComponentSetValue2(smooth_x_var_comp, "value_float", x1 + ( ( vel_x1 + additional_vel_x ) * friction_x ) )
	ComponentSetValue2(smooth_y_var_comp, "value_float", y1 + ( ( vel_y1 + additional_vel_y + gravity ) * friction_y ) )

	-- Apply angular damping

	angular_velocity = angular_velocity * angular_damping

	-- Store the updated angular velocity
	if angular_velocity_var_comp ~= nil and angular_velocity_var_comp > 0 then
		ComponentSetValue2(angular_velocity_var_comp, "value_float", angular_velocity)
	end

end